-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local C = {}
C.__index = C
local version = 1

local function numberToCurrency(num) 
  local str1 = tostring(num)

  -- 小数点以下の抽出
  local str2 = ""
  local match_s = string.find(str1, ".", 1, true)
  if match_s ~= nil then
    str2 = string.sub(str1, match_s)
    str1 = string.sub(str1, 1, match_s - 1)
  end

  -- 整数部分を後ろから3桁ずつ取り出して","で繋ぐ
  local i = 1
  str1 = string.reverse(str1)
  while (i <= string.len(str1)) do
    str2 = "," .. string.reverse(string.sub(str1, i, i + 2)) .. str2
    i = i + 3
  end

  -- 先頭に余分な","がついているので2文字目以降を返す
  return string.sub(str2, 2)
end

function C:init()
  self.latestVersion = version
  self.fgVariables = self.missionTypeData or {}
  self.fgPath = "/gameplay/missionTypes/mathkuroMission01/destruction.flow.json"

  self.starLabels = {
    bronze = 'Destruct over ' .. tostring(self.missionTypeData.bronzeScore) .. ' vehicles',
    silver = 'Destruct over ' .. tostring(self.missionTypeData.silverScore) .. ' vehicles',
    gold = 'Destruct over ' .. tostring(self.missionTypeData.goldScore) .. ' vehicles',
    totalDamage = 'Achieve damage over $' .. numberToCurrency(self.missionTypeData.totalDamage)
  }

  self.sortedStarKeys = {'bronze', 'silver', 'gold', 'totalDamage'}

  self.defaultAggregateValues = {
    all = {
      bestMedal = "none",
      bestDestroy = 0,
      bestDamages = 0
    }
  }
  self.autoAggregates = {
    {
      type = 'simpleMedal',
      attemptKey = 'medal', -- key in the attempt
      aggregateKey = 'bestMedal', -- key in the aggregate
      newBestKey = 'newBestMedal',
      default = 'none',
    },
    {
      type = 'simpleHighscore',
      attemptKey = 'destroy', -- key in the attempt
      aggregateKey = 'bestDestroy', -- key in the aggregate
      sorting = 'descending', -- keeping the higher score
      newBestKey = 'newBestDestroy',
    },
    {
      type = 'simpleHighscore',
      attemptKey = 'damages', -- key in the attempt
      aggregateKey = 'bestDamages', -- key in the aggregate
      sorting = 'descending', -- keeping the higher score
      newBestKey = 'newbestDamages',
    }
  }

  self.autoUiAttemptProgress = {
    {
      type = 'simple',
      attemptKey = 'medal',
      columnLabel = 'bigMap.progressLabels.medal',
    },
    {
      type = 'simple',
      attemptKey = 'destroy',
      columnLabel = 'Count',
    },
    {
      type = 'simple',
      attemptKey = 'damages',
      columnLabel = 'Damages[$]',
    },
  }

  self.autoUiAggregateProgress = {
    {
      type = 'simple',
      aggregateKey = 'bestMedal',
      columnLabel = 'bigMap.progressLabels.bestMedal',
      newBestKey = 'newBestMedal',
    },
    {
      type = 'simple',
      aggregateKey = 'bestDestroy',
      columnLabel = 'Best Count',
      newBestKey = 'newBestDestroy',
    },
    {
      type = 'simple',
      aggregateKey = 'bestDamages',
      columnLabel = 'Best Damages[$]',
      newBestKey = 'newbestDamages',
    }
  }

  self.autoUiBigmap = {
    aggregates = {
      aggregatePrimary = {
        progressKey = 'default',
        aggregateKey = 'bestDestroys', -- select best time from detail progress key
        label = 'Best Count',
        type = 'simple'
      }
    }
  }
  self.bigMapIcon = {icon = "mission_medal_triangle"}
  self.missionTypeLabel = 'Destruction'
end

function C:updateAttempt(attempt, newVersion) end

function C:processUserSettings(settings)
  self.userSettings = settings
end

function C:getUserSettingsData()
  local data = {}

  if self.missionTypeData.allowVehicleChoice then
    table.insert(data, {
      key = 'presetVehicleActive',
      label = 'missions.missions.general.userSettings.useProposedCar',
      type = 'bool',
      value = true
    })
  end

  if self.missionTypeData.allowTrafficDifficultyChoice then
    table.insert(data, {
      key = 'trafficDifficulty',
      label = 'Traffic Difficulty',
      type = 'select',
      values = {
        {l = 'Easy', v = 'Easy'},
        {l = 'Normal', v = 'Normal'},
        {l = 'Hard', v = 'Hard'}
      },
      value = self.missionTypeData.trafficDifficulty
    })
  end

  return data
end

function C:getRandomizedAttempt(progressKey)
  return gameplay_missions_progress.testHelper.randomAttemptType(), {
    damages = round(gameplay_missions_progress.testHelper.randomNumber(100,1000)),
    destroy = round(gameplay_missions_progress.testHelper.randomNumber(0,5)),
    medal = gameplay_missions_progress.testHelper.randomMedal()
  }
end

function C:getMissionTips()
  return {"The amount of traffic is depends on your game options. [Options] -> [GAMEPLAY] -> [Traffic]"}
end

return function(...) return gameplay_missions_missions.flowMission(C, ...) end
