angular
.module('beamng.apps')
.controller('AppCtrl', function ($scope, $mdSidenav) {
  // ---- 定数 ----

  // 言語設定
  const DEFAULT_LANG = 'en';
  const SUPPORT_LANGS = ['en', 'ja'];
  let currentLanguage = DEFAULT_LANG;

  let isFreeroam = false;

  // 近くの車両のダメージを画面表示する設定
  let damagedVehicleDisplayTimerMap = new Map();
  const DISPLAY_DAMAGE_THRESHOLD = 100;
  const DISPLAY_FRAMES = 120;
  // 画面更新頻度
  const DAMAGE_VALUE_UPDATE_RATE = 5;
  let update_rate_count = 0;

  // 被害情報
  let currentTotalDamage = 0;
  let damagedVehicleNum = 0;

  // 画面に表示する変数の初期化
  $scope.totalDamage = 0;
  $scope.totalDamageTitle = '';
  $scope.damagedVehicleNum = 0;
  $scope.damagedVehicleNumTitle = '';
  $scope.nearVehicleDamages = [];
  $scope.ja_jp = false;

  // Get user language settings. ('<lang>_<country>')
  // ref: 0.25\lua\common\utils\languageMap.lua
  bngApi.engineLua('Lua:getSelectedLanguage()', (userLang) => {
    let _lang = userLang.split('_')[0];
    currentLanguage = (SUPPORT_LANGS.includes(_lang)) ? _lang : DEFAULT_LANG;
    if (currentLanguage == "ja") {
      $scope.ja_jp = true;
    }
    console.log('set language: ' + currentLanguage);
  });

  // for freeroam mode
  bngApi.engineLua('core_gamestate.state.state', (state) => {
    if (state == "freeroam") {
      isFreeroam = true;
      bngApi.engineLua("extensions.unload('mathkuro_missionDestruction')");
      bngApi.engineLua("extensions.loadAtRoot('lua/ge/extensions/mathkuro/missionDestruction', 'mathkuro')");
    }
  });

  // フレーム毎のループ
  $scope.$on('streamsUpdate', function (event, data) {
    // 指定言語に合わせて表示に変更
    let locale = getLocale(currentLanguage);

    update_rate_count = update_rate_count + 1;
    if (0 == update_rate_count % DAMAGE_VALUE_UPDATE_RATE) {
      update_rate_count = 0;

      if (isFreeroam) {
        // ダメージ情報更新(ミッション中はFlowgraph内で更新)
        bngApi.engineLua('mathkuro_missionDestruction.updateDamagedVehicleTable()');
      }

      // 被害総額取得
      bngApi.engineLua('mathkuro_missionDestruction.getDamagedVehicleNum()', (_num) => {
        damagedVehicleNum = _num;
      });

      // 被害総額取得
      bngApi.engineLua('mathkuro_missionDestruction.getTotalDamageValue()', (damage) => {
        currentTotalDamage = damage;
      });

      // 車両毎の被害額取得
      bngApi.engineLua('mathkuro_missionDestruction.getDamagedVehicleTable()', (_damagedVehicleTable) => {
        Object.keys(_damagedVehicleTable).forEach((key) => {
          let _vname = _damagedVehicleTable[key]['vname'];
          let _damageValue = _damagedVehicleTable[key]['damageValue'];
          let _last_val = damagedVehicleDisplayTimerMap.get(key);
          if (typeof _last_val === 'undefined') {
            // 表示数が増えて見にくくなってしまうため、ダメージ値が一定以上の被害車両のみ追加
            if (_damageValue >= DISPLAY_DAMAGE_THRESHOLD) {
              damagedVehicleDisplayTimerMap.set(key, {vname: _vname, damageValue: _damageValue, remain: DISPLAY_FRAMES});
            }
          } else {
            // UPDATE
            if (_last_val['remain'] <= 0 && _damageValue < _last_val['damageValue']) {
              damagedVehicleDisplayTimerMap.delete(key)
            } else {
              _last_val['damageValue'] = _damageValue;
              damagedVehicleDisplayTimerMap.set(key, _last_val);
            }
          }
        });
      });
    }

    // 画面表示更新
    $scope.$evalAsync(function() {
      let _damages = [];
      damagedVehicleDisplayTimerMap.forEach((val, key) => {
        val['remain'] = val['remain'] - 1;
        if (val['remain'] > 0) {
          // 表示期間が残っている場合のみ画面表示
          // console.log('name: ' + val['vname'] + ', damageValue:' + val['damageValue'] + ', remain:' + val['remain']);
          _damages.push({name: val['vname'], damageValue: locale.damageToLocalString(val['damageValue'])});
        }
      });

      $scope.totalDamageTitle = locale.totalDamageTitle;
      $scope.totalDamage = locale.damageToLocalString(currentTotalDamage);
      $scope.damagedVehicleNumTitle = locale.damagedVehicleNumTitle;
      $scope.damagedVehicleNum = damagedVehicleNum
      $scope.nearVehicleDamages = _damages;
    });
  });

  // ---- UI上に表示する言語の設定 ----
  function getLocale(localeName){
    if (localeName == 'ja') {
      return new JaLocale();
    } else {
      return new EnLocale();
    }
  };

  class LocalInfoBase {
    totalDamageTitle = 'CRASH TOTAL';
    damagedVehicleNumTitle = 'WRECKS';

    damageToLocalString(damageValue) {
      return '$' + damageValue.toLocaleString(undefined, { maximumFractionDigits: 0 });
    }
  };

  class EnLocale extends LocalInfoBase {};

  class JaLocale extends LocalInfoBase {
    totalDamageTitle = '被害総額';
    damagedVehicleNumTitle = '被害台数';

    // 京の桁まで対応
    maxDisplayDigits = 10000 * 10000 * 10000 * 10000 * 10000;

    damageToLocalString(damageValue) {
      let _damage = damageValue * 120;

      if (_damage > this.maxDisplayDigits) {
        return '￥' + (damageValue * 120).toLocaleString(undefined, { maximumFractionDigits: 0 });
      } else {
        let _str = String(Math.round(_damage));
        let keta = ['', '万', '億', '兆', '京'];
        let nums = _str.replace(/(\d)(?=(\d\d\d\d)+$)/g, '$1,').split(',').reverse();
        let data = '';
        for (let i = 0; i < nums.length; i++) {
          if ((nums.length - i) > 2) {
            continue;
          }

          if (!nums[i].match(/^[0]+$/)) {
            data = nums[i].replace(/^[0]+/g, '') + keta[i] + data;
          }
        }
        if (data == '') {
          data = '0';
        }
        return data + '円';
      }
    }
  };

})
.directive('destructionMission', [function () {
  return {
    templateUrl: '/ui/modules/apps/destructionMissionMathkuro/app.html',
    replace: true,
    restrict: 'EA',
    link: function (scope, element, attrs) {
      'use strict';
    }
  };
}])
